/*
 * Copyright (C) 2004  Heinz Mauelshagen, Red Hat GmbH. All rights reserved.
 *
 * See file LICENSE at the top of this source tree for license information.
 */

#include <string.h>
#include <sys/ioctl.h>
#include <mm/dbg_malloc.h>
#include "dev-io.h"
#include "scsi.h"

/* Ioctl the SCSI serial number off a unit (thx scsiinfo). */
int get_scsi_serial(struct disk_info **di, int fd)
{
	int ret = 0;
	size_t size;
	unsigned char *buffer, *c, *pagestart, *str;
	unsigned int *ibuffer;
#define	SIZE	1024 + 4 * sizeof(*ibuffer) /* Should be large enough. */

	if (!(buffer = dbg_malloc(SIZE)))
		return 0;

	memset(buffer, 0, SIZE);
	ibuffer = (int *) buffer;
	ibuffer[0] = 0;		/* input data length */
	ibuffer[1] = 1024;	/* output buffer length */
	c = (char *) &ibuffer[2];
	c[0] = 0x12;		/* INQUIRY */
	c[1] = 0x01;		/* lun=0, evpd=1 */
	c[2] = 0x80;		/* page code = 0x80, serial number */
	c[3] = 0x00;		/* (reserved) */
	c[4] = 0xff;		/* allocation length */
	c[5] = 0x00;		/* control */
	
	/* SCSI_IOCTL_SEND_COMMAND */
	if (ioctl(fd, 1, buffer))
		goto free;
	
	pagestart = buffer + 8;
	size = pagestart[3];
	str  = &pagestart[4];

	if (((*di)->serial = dbg_strdup(remove_spaces(str, size))))
		ret = 1;

  free:
	dbg_free(buffer);

	return ret;
}
