/*
 * Copyright (C) 2004  Heinz Mauelshagen, Red Hat GmbH. All rights reserved.
 *
 * See file LICENSE at the top of this source tree for license information.
 */

#include <stdarg.h>
#include "internal.h"
#include "version.h"

static void init_cmd(struct lib_context *lc, char **argv)
{
	lc->cmd = strrchr(argv[0], '/');
	lc->cmd = lc->cmd ? lc->cmd + 1 : argv[0];
}

static void init_version(struct lib_context *lc)
{
	lc->version.text	= DMRAID_LIB_VERSION;

	lc->version.v.major	= DMRAID_LIB_MAJOR_VERSION;
	lc->version.v.minor	= DMRAID_LIB_MINOR_VERSION;
	lc->version.v.sub_minor	= DMRAID_LIB_SUBMINOR_VERSION;
	lc->version.v.suffix	= DMRAID_LIB_VERSION_SUFFIX;
}

struct lib_context *alloc_lib_context(char **argv)
{
	int i;
	struct lib_context *lc;

	if ((lc = dbg_malloc(sizeof(*lc)))) {
		memset(lc, 0, sizeof(*lc));
		init_cmd(lc, argv);
		init_version(lc);
		lc->mode = 0600;
		for (i = 0; i < LC_LISTS_SIZE; i++)
			INIT_LIST_HEAD(&lc->lists[i]);
	} else
		log_err(lc, "allocting library context");

	return lc;
}

void free_lib_context(struct lib_context *lc)
{
	dbg_free(lc);
}

int lc_opt(struct lib_context *lc, enum lc_options o)
{
	return lc->opts[o];
}

int lc_inc_opt(struct lib_context *lc, int o)
{
	return ++lc->opts[o];
}

struct list_head *lc_list(struct lib_context *lc, int l)
{
	return l < ARRAY_SIZE(lc->lists) ? &lc->lists[l] : NULL;
}
