/*
 * Copyright (C) 2004  Heinz Mauelshagen, Red Hat GmbH. All rights reserved.
 *
 * See file LICENSE at the top of this source tree for license information.
 */

#include <stdarg.h>
#include "internal.h"

/* Prompt for a yes/no answer */
int yes_no_prompt(const char *prompt, ...)
{
	int c = 0;
	va_list ap;

	while (c != 'y' && c != 'n') {
		if (c == '\n' || c == 0) {
			va_start(ap, prompt);
			vprintf(prompt, ap);
			va_end(ap);
			printf(" ? [y/n] :");
		}
		c = tolower(getchar());
	}

	while (getchar() != '\n') ;

	return (c == 'y') ? 1 : 0;
}

/* Skip any leading and remove any trailing blanks from a string. */
char *remove_spaces(char *str, size_t size)
{
	char *s = &str[size-1];

	/* Work on trailing blanks. */
	while (isspace(*s))
		s--;

	*(s+1) = '\0';

	/* Work on leading blanks. */
	s = str;
	while (isspace(*s))
		s++;

	return s;
}

/* Convert a numeric string to alpha. */
void mk_alpha(char *str, size_t len)
{
	char *s;
	static char asc[] = "abcdefghij";
	static char buf[2] = { '\0', '\0' };

	for (s = str; *s && len; s++, len--) {
		if (isdigit(*s)) {
			buf[0] = *s;
			*s = asc[atoi(buf)];
		}
	}
}

/* Grow a string. */
static int grow_string(char **string, const char *s)
{
	size_t len;
	char *tmp = *string;

 	len = strlen(s) + (tmp ? strlen(tmp) + 1 : 1);
	if ((*string = dbg_realloc(*string, len))) {
		if (!tmp)
			**string = '\0';
	} else if (tmp)
		dbg_free(tmp);
	
	return *string ? 1 : 0;
}

/* Free a string. */
void free_string(char **string)
{
	if (*string) {
		dbg_free(*string);
		*string = NULL;
	}
}

/* Push a string onto the end of another. */
int p_str(char **string, const char *s)
{
	if (grow_string(string, s)) {
		strcat (*string, s);
		return 1;
	}

	return 0;
}

/* Push an uint64_t in ascii onto the end of a string. */
int p_u64(char **string, uint64_t u)
{
	char buffer[22];

	sprintf(buffer, "%" PRIu64, u);
	if (grow_string(string, buffer)) {
		sprintf(&(*string)[strlen(*string)], "%" PRIu64, u);
		return 1;
	}

	return 0;
}
